#include "pop3.h"

Pop3::Pop3(char *hostName, int portNumber, char *inUserId, char *inPasswd,
	char *inEmailDir, bool inLeaveOnServer) : Sockets(hostName, portNumber)
{
	userId = new char[strlen(inUserId) + 1];
	strcpy(userId, inUserId);
	
	passwd = new char[strlen(inPasswd) + 1];
	strcpy(passwd, inPasswd);
	
	emailDir = new char[strlen(inEmailDir) + 1];
	strcpy(emailDir, inEmailDir);
	
	leaveOnServer = inLeaveOnServer;
}

Pop3::~Pop3()
{
	delete[] userId;
	delete[] passwd;
	delete[] emailDir;
}

bool Pop3::sendUserId()
{
	char *buffer = new char[strlen(userId) + 8];
	sprintf(buffer, "USER %s\r\n", userId);
	cout << buffer;
	if(!sendStream(buffer))
	{
		delete[] buffer;
		return 0;
	}
	delete[] buffer;

	if(!recvCommand())
		return 0;

	return 1;
}	

bool Pop3::sendPasswd()
{	
	char *buffer = new char[strlen(passwd) + 8];
	sprintf(buffer, "PASS %s\r\n", passwd);
	cout << "PASS ********\r\n";
	if(!sendStream(buffer))
	{
		delete[] buffer;
		return 0;
	}
	delete[] buffer;

	if(!recvCommand())
		return 0;
		
	return 1;
}

int Pop3::getNumMessages()
{
	if(!sendStream("STAT\r\n"))
		return -1;
	cout << "STAT\r\n";

	char *buffer = new char[RECV_BUFFER_LENGTH + 1];
	if(!recvStream(buffer, RECV_BUFFER_LENGTH))	
	{
		delete[] buffer;
		return 0;
	}
	else if(!checkOK(buffer))
	{
		cout << buffer;
		delete[] buffer;
		return 0;
	}
	
	cout << buffer;
		
	int numMessages;
	sscanf(&buffer[4], "%i", &numMessages);
	
	delete[] buffer;	
	
	return numMessages;
}

ssize_t Pop3::getSizeMessage(int messageNum)	
{
	ssize_t messageSize;

	char *buffer = new char[8 + intLength(messageNum)];
	sprintf(buffer, "LIST %i\r\n", messageNum);

	if(!sendStream(buffer))
	{
		delete[] buffer;
		return 0;
	}
	cout << buffer;

	delete[] buffer;
	
	buffer = new char[RECV_BUFFER_LENGTH + 1];
	if(!recvStream(buffer, RECV_BUFFER_LENGTH))	
	{
		delete[] buffer;
		return 0;
	}
	else if(!checkOK(buffer))
	{
		cout << buffer;
		delete[] buffer;
		return 0;
	}
	cout << buffer;
	
	sscanf(&buffer[5 + intLength(messageNum)], "%ld", &messageSize);

	return messageSize;
}

bool Pop3::retrMessages()
{
	int numMessages;
	if(!(numMessages = getNumMessages()))
		return 0;

	for(int k = 1; k <= numMessages; k++)
	{
		ssize_t messageSize = getSizeMessage(k);
	
		char *buffer = new char[8 + intLength(k)];
		sprintf(buffer, "RETR %i\r\n", k);
		
		if(!sendStream(buffer))
		{
			delete[] buffer;
			return 0;
		}
		cout << buffer;
		
		delete[] buffer;
		
		if(!recvCommand())
			return 0;
		if(!recvMessage(messageSize))
			return 0;
	}

	return 1;
}

bool Pop3::deleMessages()
{
	if(leaveOnServer)
		return 1;
	
	int numMessages;
	if(!(numMessages = getNumMessages()))
		return 0;

	for(int k = 1; k <= numMessages; k++)
	{
		char *buffer = new char[8 + intLength(k)];
		sprintf(buffer, "DELE %i\r\n", k);
		
		if(!sendStream(buffer))
		{
			delete[] buffer;
			return 0;
		}
		cout << buffer;
		
		delete[] buffer;
		
		recvCommand();
	}

	return 1;
}

bool Pop3::recvCommand()
{
	char *buffer = new char[RECV_BUFFER_LENGTH + 1];
	if(!recvStream(buffer, RECV_BUFFER_LENGTH))	
	{
		delete[] buffer;
		return 0;
	}
	else if(!checkOK(buffer))
	{
		cout << buffer;
		delete[] buffer;
		return 0;
	}
	cout << buffer;
	
	delete[] buffer;
	
	return 1;
}

bool Pop3::recvMessage(ssize_t messageSize)
{
	char *entireMessage = new char[messageSize + 1];
	char *buffer = new char[RECV_BUFFER_LENGTH + 1];

	while(streamWaiting())
	{
		recvStream(buffer, RECV_BUFFER_LENGTH);
		strcat(entireMessage, buffer);
	}

	delete[] buffer;
	
	Parser p1(entireMessage, emailDir);

	delete[] entireMessage;

	return 1;
}

bool Pop3::checkOK(char *recvBuffer)
{
	if(!(strncmp(recvBuffer, "+OK", 3)))
		return 1;
	else
		return 0;
}

int Pop3::intLength(int num)
{
	if(num < 10)
		return 1;
	else if(num < 100)
		return 2;
	else if(num < 1000)
		return 3;
	else if(num < 10000)
		return 4;
	cout << "Error: intLength() out of bounds, You have way too many messages :)\n";
	exit(-1);
}

bool Pop3::quit()
{
	if(!sendStream("QUIT\r\n"))
		return 0;
	cout << "QUIT\r\n";

	recvCommand();
		
	return 1;
}

